/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls/cpp/ftpsearch.hpp
 *
 * Purpose:     recls C++ mapping - FtpSearch class.
 *
 * Created:     18th August 2003
 * Updated:     14th March 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_RECLS_CPP_HPP_FTPSEARCH
#define RECLS_INCL_RECLS_CPP_HPP_FTPSEARCH

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_RECLS_CPP_HPP_FTPSEARCH_MAJOR    2
# define RECLS_VER_RECLS_CPP_HPP_FTPSEARCH_MINOR    1
# define RECLS_VER_RECLS_CPP_HPP_FTPSEARCH_REVISION 2
# define RECLS_VER_RECLS_CPP_HPP_FTPSEARCH_EDIT     7
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls/cpp/ftpsearch.hpp Definition of the \link recls::cpp::FileSearch FtpSearch\endlink class, part of 
 * the \ref group_recls_cpp "recls C++ mapping"
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls/cpp/recls.hpp>
#include <recls/cpp/search.hpp>
#include <recls_ftp.h>
#include <recls_assert.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{

namespace cpp
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// This class provides functionality for executing a file search
///
/// \ingroup group_recls_cpp
class FtpSearch
    : public Search
{
/// \name Types
public:
/// @{
    typedef FtpSearch   class_type;
/// @}

/// \name Construction
/// @{
public:
    /// Creates a search for the items identified by \c pattern in the directory \c rootDir, according to the given \c flags.
    ///
    /// \param host The host on which to search
    /// \param username The username with which to gain access
    /// \param password The password with which to gain access
    /// \param rootDir The starting directory for the search. If NULL, or the empty string, uses the current directory
    /// \param pattern The search pattern, e.g. "*.h". If NULL, or the empty string, uses "*.*" (Win32) or "*" (UNIX)
    /// \param flags A combination of enumerants from the \c RECLS_FLAG enumeration
    FtpSearch(recls_char_t const *host, recls_char_t const *username, recls_char_t const *password, recls_char_t const *rootDir, recls_char_t const *pattern, recls_uint32_t flags);
    /// Destructor
    ~FtpSearch();
/// @}

// Implementation
private:
    static ctor_args_type CreateSearch_(recls_char_t const *host, recls_char_t const *username, recls_char_t const *password, char const *rootDir, char const *pattern, recls_uint32_t flags)
    {
        hrecls_t    hSrch;
        recls_rc_t  rc = Recls_SearchFtp(host, username, password, rootDir, pattern, flags, &hSrch);

        return Search::ctor_args_type(hSrch, rc);
    }

// Not to be implemented
private:
    FtpSearch(class_type const &rhs);
    class_type &operator =(class_type const &rhs);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

inline FtpSearch::FtpSearch(recls_char_t const *host, recls_char_t const *username, recls_char_t const *password, recls_char_t const *rootDir, recls_char_t const *pattern, recls_uint32_t flags)
    : Search(CreateSearch_(host, username, password, rootDir, pattern, flags))
{}

inline FtpSearch::~FtpSearch()
{}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace cpp */
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_RECLS_CPP_HPP_FTPSEARCH */

/* ////////////////////////////////////////////////////////////////////////// */
