/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_retcodes.h
 *
 * Purpose:     Return codes for the  recls API.
 *
 * Created:     15th August 2003
 * Updated:     28th March 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#if !defined(RECLS_INCL_H_RECLS) && \
    !defined(RECLS_DOCUMENTATION_SKIP_SECTION)
# error recls_retcodes.h cannot be included directly. Include recls.h
#else

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_RETCODES_MAJOR       2
# define RECLS_VER_H_RECLS_RETCODES_MINOR       1
# define RECLS_VER_H_RECLS_RETCODES_REVISION    3
# define RECLS_VER_H_RECLS_RETCODES_EDIT        21
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_retcodes.h Return codes for the \ref group_recls  API */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Helper macros
 */

#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_CAST_OPERATOR_TRIGRAPH_SPLITTER  /* Just to make doubly sure that we can't get a <:: trigraph from a globally qualified type */
# ifdef __cplusplus
#  define _RECLS_STATIC_CAST(t, v)              (static_cast<RECLS_CAST_OPERATOR_TRIGRAPH_SPLITTER t>(v))
# else /* ? __cplusplus */
#  define _RECLS_STATIC_CAST(t, v)              ((t)(v))
# endif /* __cplusplus */
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Error codes
 */

/** \name Error codes
 *
 * \ingroup group_recls
 */
/** @{ */

/** No search is currently active */
#define RECLS_RC_SEARCH_NO_CURRENT          _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1001)
/** The directory was invalid, or does not exist */
#define RECLS_RC_INVALID_DIRECTORY          _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1002)
/** No more data is available */
#define RECLS_RC_NO_MORE_DATA               _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1003)
/** Memory exhaustion */
#define RECLS_RC_OUT_OF_MEMORY              _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1004)
/** Function not implemented */
#define RECLS_RC_NOT_IMPLEMENTED            _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1005)
/** Invalid search type */
#define RECLS_RC_INVALID_SEARCH_TYPE        _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1006)
/** Invalid search protocol */
#define RECLS_RC_INVALID_SEARCH_PROTOCOL    _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1007)
/** Unexpected internal condition */
#define RECLS_RC_UNEXPECTED                 _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1008)
/** Cannot specify "." pattern in recursive search */
#define RECLS_RC_DOT_RECURSIVE_SEARCH       _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1009)
/** Search was cancelled by callback. It did not otherwise fail. */
#define RECLS_RC_SEARCH_CANCELLED           _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1010)
/** FTP infrastructure initialisation failed. */
#define RECLS_RC_FTP_INIT_FAILED            _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1011)
/** FTP connection failed. */
#define RECLS_RC_FTP_CONNECTION_FAILED      _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1012)
/** Given entry was of wrong type */
#define RECLS_RC_WRONG_ENTRY_TYPE           _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1013)
/** Cannot call stat() on root */
#define RECLS_RC_CANNOT_STAT_ROOT           _RECLS_STATIC_CAST(RECLS_QUAL(recls_rc_t), -1 - 1014)

/** @} */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS */

/* ////////////////////////////////////////////////////////////////////////// */
