////////////////////////////////////////////////////////////////////////////////
// EnumEntry.cpp : Implementation of EnumEntry
//
// Created: 
// Updated: 8th February 2004
//

#include "stdafx.h"
#include "Recls_com.h"
#include "EnumEntry.h"
#include "FileEntry.h"

/////////////////////////////////////////////////////////////////////////////
// EnumEntry

/* static */ HRESULT WINAPI EnumEntry::QueryFunc(void *pv, REFIID riid, LPVOID *ppv, DWORD dw)
{
	EnumEntry	*peev	=	(EnumEntry*)pv;

	if(riid == IID_IEnumVARIANT)
	{
		if(peev->m_bIEnumFileEntry)
		{
			*ppv = NULL;

			return E_NOINTERFACE;
		}
		else
		{
			peev->m_bIEnumVARIANT = true;
			*ppv = static_cast<IEnumVARIANT*>(peev);
			static_cast<LPUNKNOWN>(*ppv)->AddRef();

			return S_OK;
		}
	}
	else if(riid == IID_IEnumFileEntry)
	{
		if(peev->m_bIEnumVARIANT)
		{
			*ppv = NULL;

			return E_NOINTERFACE;
		}
		else
		{
			peev->m_bIEnumFileEntry = true;
			*ppv = static_cast<IEnumFileEntry*>(peev);
			static_cast<LPUNKNOWN>(*ppv)->AddRef();

			return S_OK;
		}
	}
	else
	{
		ATLTRACE("Unknown interface ID requested\n");
		ATLASSERT(0);
	}

	return E_UNEXPECTED;
}


inline HRESULT EnumEntry::DoSkip()
{
 	ATLASSERT(!m_info.bEmpty);
	ATLASSERT(NULL != m_info.hSrch);

	HRESULT		hr;
	recls_rc_t	rc	=	Recls_GetNext(m_info.hSrch);

	if(RECLS_RC_NO_MORE_DATA == rc)
	{
		hr = S_FALSE;
	}
	else if(RECLS_FAILED(rc))
	{
		set_error_info_from_recls_rc_t(rc);

		hr = E_FAIL;
	}
	else
	{
		hr = S_OK;
	}

	return hr;
}

inline HRESULT EnumEntry::DoNext(VARIANT *pvar)
{
 	ATLASSERT(!m_info.bEmpty);
	ATLASSERT(!m_bIEnumFileEntry);

	HRESULT		hr;

	if(NULL == m_info.hSrch)
	{
		hr = S_FALSE;
	}
	else
	{
		info_t		entry;
		recls_rc_t	rc	=	Recls_GetDetails(m_info.hSrch, &entry);

		if(RECLS_RC_NO_MORE_DATA == rc)
		{
			hr = S_FALSE;
		}
		else if(RECLS_FAILED(rc))
		{
			set_error_info_from_recls_rc_t(rc);

			hr = E_FAIL;
		}
		else
		{
			// The entry is available, so create an Entry, and place it
			// in the variant
			::VariantInit(pvar);

			FileEntry	*pEntry	=	new CComObject<FileEntry>;

			if(NULL == pEntry)
			{
				Recls_CloseDetails(entry);

				hr = E_OUTOFMEMORY;
			}
			else
			{
				pEntry->AddRef();
				pEntry->SetVoid(entry);

				pvar->vt		=	VT_DISPATCH;
				pvar->pdispVal	=	pEntry;

				// Now test the next spot
				if(RECLS_FAILED(Recls_GetNext(m_info.hSrch)))
				{
					Recls_SearchClose(m_info.hSrch);
					m_info.hSrch = NULL;
				}

				hr = S_OK;
			}
		}
	}

	return hr;
}

inline HRESULT EnumEntry::DoNext(PFILEENTRY *ppe)
{
 	ATLASSERT(!m_info.bEmpty);
	ATLASSERT(!m_bIEnumVARIANT);

	HRESULT		hr;

	if(NULL == m_info.hSrch)
	{
		hr = S_FALSE;
	}
	else
	{
		info_t		entry;
		recls_rc_t	rc	=	Recls_GetDetails(m_info.hSrch, &entry);

		if(RECLS_RC_NO_MORE_DATA == rc)
		{
			hr = S_FALSE;
		}
		else if(RECLS_FAILED(rc))
		{
			set_error_info_from_recls_rc_t(rc);

			hr = E_FAIL;
		}
		else
		{
			// The entry is available, so create an Entry, and place it
			// in the variant
			*ppe = NULL;

			FileEntry	*pEntry	=	new CComObject<FileEntry>;

			if(NULL == pEntry)
			{
				Recls_CloseDetails(entry);

				hr = E_OUTOFMEMORY;
			}
			else
			{
				pEntry->AddRef();
				pEntry->SetVoid(entry);

				*ppe	=	pEntry;

				// Now test the next spot
				if(RECLS_FAILED(Recls_GetNext(m_info.hSrch)))
				{
					Recls_SearchClose(m_info.hSrch);
					m_info.hSrch = NULL;
				}

				hr = S_OK;
			}
		}
	}

	return hr;
}

// IEnumXXXX

STDMETHODIMP EnumEntry::Skip(/* [in] */ ULONG celt)
{
	HRESULT	hr;

	if(m_info.bEmpty)
	{
		hr = S_FALSE;
	}
	else
	{
		for(hr = S_OK; celt > 0 && S_OK == (hr = DoSkip()); --celt)
		{}
	}

	return hr;
}

STDMETHODIMP EnumEntry::Reset()
{
	HRESULT	hr;

	// Close the current search ...
	Recls_SearchClose(m_info.hSrch);

	// ... and start another one
#ifdef RECLS_CHAR_TYPE_IS_CHAR
	recls_rc_t	rc	=	Recls_Search(winstl::w2a(m_info.searchRoot), winstl::w2a(m_info.pattern), static_cast<recls_uint32_t>(m_info.flags), &m_info.hSrch);
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
	recls_rc_t	rc	=	Recls_Search(m_info.searchRoot, m_info.pattern, static_cast<recls_uint32_t>(m_info.flags), &m_info.hSrch);
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */

	if(RECLS_FAILED(rc))
	{
		set_error_info_from_recls_rc_t(rc);

		hr = E_FAIL;
	}
	else
	{
		hr = S_OK;
	}

	return hr;
}

// IEnumVARIANT

STDMETHODIMP EnumEntry::Next(	/* [in] */ ULONG celt,
								/* [length_is][size_is][out] */ VARIANT *rgVar,
								/* [out] */ ULONG *pceltFetched)
{
	HRESULT	hr;
	ULONG	celtFetched;

	ATLASSERT(!m_bIEnumFileEntry);

	if(NULL == pceltFetched)
	{
		pceltFetched = &celtFetched;
	}

	if(m_info.bEmpty)
	{
		celtFetched = 0;

		hr = S_FALSE;
	}
	else
	{
		for(hr = S_OK, celtFetched = 0; celt > 0 && S_OK == (hr = DoNext(rgVar)); --celt, ++rgVar, ++celtFetched)
		{}

		if(FAILED(hr))
		{
			hr = S_FALSE;
		}
	}

	*pceltFetched = celtFetched;

	return hr;
}

STDMETHODIMP EnumEntry::Clone(/* [out] */ IEnumVARIANT **ppEnum)
{
	ATLASSERT(!m_bIEnumFileEntry);

	return (NULL == ppEnum) ? E_POINTER : (*ppEnum = NULL, E_FAIL);
}

// IEnumFileEntry

STDMETHODIMP EnumEntry::Next(	/* [in] */ ULONG celt,
								/* [length_is][size_is][out] */ PFILEENTRY *rgVar,
								/* [out] */ ULONG *pceltFetched)
{
	HRESULT	hr;
	ULONG	celtFetched;

	ATLASSERT(!m_bIEnumVARIANT);

	if(NULL == pceltFetched)
	{
		pceltFetched = &celtFetched;
	}

	if(m_info.bEmpty)
	{
		celtFetched = 0;

		hr = S_FALSE;
	}
	else
	{
		for(hr = S_OK, celtFetched = 0; celt > 0 && S_OK == (hr = DoNext(rgVar)); --celt, ++rgVar, ++celtFetched)
		{}

		if(FAILED(hr))
		{
			hr = S_FALSE;
		}
	}

	*pceltFetched = celtFetched;

	return hr;
}

STDMETHODIMP EnumEntry::Clone(/* [out] */ IEnumFileEntry **ppEnum)
{
	ATLASSERT(!m_bIEnumVARIANT);

	return (NULL == ppEnum) ? E_POINTER : (*ppEnum = NULL, E_FAIL);
}

////////////////////////////////////////////////////////////////////////////////
