/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_Entry.cpp
 *
 * Purpose:     JNI implementation file for the Entry class
 *
 * Created      21st November 2003
 * Updated:     19th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include "recls_jni_util.h"

#include <stlsoft/auto_buffer.hpp>
#include <stlsoft/malloc_allocator.hpp>

/* ////////////////////////////////////////////////////////////////////////// */

typedef	stlsoft_ns_qual(auto_buffer)<	recls_char_t
									,	stlsoft_ns_qual(malloc_allocator)<recls_char_t>
									,	512
									>		buffer_t;

/* ////////////////////////////////////////////////////////////////////////// */

void ThrowException(JNIEnv *henv, char const *className, char const *message)
{
	va_list	args;

	va_start(args, message);

	jclass	clsX	=	henv->FindClass(className);

	if(NULL != clsX)
	{
		henv->ThrowNew(clsX, message);

		henv->DeleteLocalRef(clsX);
	}

	va_end(args);
}

void ThrowReclsException(JNIEnv *henv, char const *message, recls_rc_t code)
{
	jclass		clsX		=	henv->FindClass(CLASSNAME_RECLSX);
	jmethodID	midXCtor	=	henv->GetMethodID(clsX, "<init>", "(Ljava/lang/String;I)V");
	jstring		jsMessage	=	henv->NewStringUTF(message);
	jobject		x			=	henv->NewObject(clsX, midXCtor, jsMessage, code);

	henv->Throw(static_cast<jthrowable>(x));

	henv->DeleteLocalRef(clsX);
}

jstring StringFromRange(JNIEnv *henv, recls_char_t const *from, recls_char_t const *to)
{
	buffer_t	buffer(1 + (to - from));

#ifdef RECLS_CHAR_TYPE_IS_CHAR
	strncpy(&buffer[0], from, buffer.size() - 1);
	buffer[buffer.size() - 1] = 0;

	return henv->NewStringUTF(&buffer[0]);
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR
	wcsncpy(&buffer[0], from, buffer.size() - 1);
	buffer[buffer.size() - 1] = 0;

	return henv->NewString(&buffer[0]);
#else /* ?char type */
# error recls char type must be discriminated
#endif /* char type */
}

jsize SafeStringUTFLength(JNIEnv *henv, jstring str)
{
	return NULL == str ? 0 : henv->GetStringUTFLength(str);
}

jsize SafeStringLength(JNIEnv *henv, jstring str)
{
	return NULL == str ? 0 : henv->GetStringLength(str);
}

char const *SafeStringUTFChars(JNIEnv *henv, jstring str, jboolean *isCopy)
{
	return NULL == str ? "" : henv->GetStringUTFChars(str, isCopy);
}

jchar const *SafeStringChars(JNIEnv *henv, jstring str, jboolean *isCopy)
{
	return NULL == str ? L"" : henv->GetStringChars(str, isCopy);
}

/* ////////////////////////////////////////////////////////////////////////// */
