/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsFileSearchDirectoryNode_unix.cpp
 *
 * Purpose:     Implementation of the ReclsFileSearchDirectoryNode class for UNIX.
 *
 * Created:     31st May 2004
 * Updated:     19th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#error This file is now obsolete, as of version 1.6.1. All content is moved to ReclsFileSearchDirectoryNode.cpp

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls.h>
#include <recls_assert.h>
#include "recls_string.h"
#include "recls_util.h"
#include "recls_impl.h"

#if defined(RECLS_CHAR_TYPE_IS_WCHAR)
# if defined(RECLS_PLATFORM_IS_WIN32)
#  include <winstl/char_conversions.hpp>
# endif /* RECLS_PLATFORM_IS_WIN32 */
#endif /* RECLS_CHAR_TYPE_IS_???? */

#include "recls_debug.h"

#include "ReclsFileSearchDirectoryNode.hpp"
#include "EntryFunctions.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

namespace {

typedef ReclsFileSearchDirectoryNode::traits_type               traits_type;
typedef ReclsFileSearchDirectoryNode::file_path_buffer_type     file_path_buffer_type;

stlsoft_ns_using(c_str_ptr)

} // anonymous namespace

/* /////////////////////////////////////////////////////////////////////////////
 * Constants
 */

static const recls_char_t   s_localDir[]                =   RECLS_LITERAL(".");

/* /////////////////////////////////////////////////////////////////////////////
 * Utility functions
 */

static size_t count_dir_parts(recls_char_t const *begin, recls_char_t const *end)
{
    return count_char_instances(begin, end, traits_type::path_name_separator());
}

/// Creates an entry from the platform-specific record

#if defined(RECLS_PLATFORM_IS_UNIX)
static recls_info_t create_entryinfo_from_psrecord(recls_char_t const * /* rootDir */, recls_uint32_t flags, recls_char_t const *entry);

static recls_info_t create_entryinfo_from_psrecord_(recls_char_t const * /* rootDir */, recls_uint32_t flags, recls_char_t const *entry)
{
    file_path_buffer_type   rootDir;
    file_path_buffer_type   entryName;
    recls_char_t            *name_start;

    traits_type::str_copy(&rootDir[0], entry);
    name_start = strrchr(&rootDir[0], RECLS_LITERAL('/'));
    traits_type::str_copy(&entryName[0], ++name_start);
    *name_start = '\0';

    return create_entryinfo_from_psrecord(c_str_ptr(rootDir), flags, c_str_ptr(entryName));
}

static recls_info_t create_entryinfo_from_psrecord(recls_char_t const *rootDir, recls_uint32_t flags, recls_char_t const *entry)
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */
{
    function_scope_trace("create_entryinfo_from_psrecord");

    // size of structure is:
    //
    //    offsetof(struct recls_fileinfo_t, data)
    //  + directory parts
    //  + full path (+ null)
    //  + short name (+ null)

    const size_t            cchRootDir  =   traits_type::str_len(rootDir);
#if defined(RECLS_PLATFORM_IS_UNIX)
    const char_type         *entryName  =   entry;
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */
    char_type const *const  dir0        =   find_directory_0(rootDir);
    char_type const *const  end         =   &rootDir[cchRootDir];

    recls_assert(cchRootDir > 0);
    recls_assert(rootDir[cchRootDir - 1] == traits_type::path_name_separator());

    const size_t    cchFileName =   traits_type::str_len(entryName);
    const size_t    cDirParts   =   (RECLS_F_DIRECTORY_PARTS == (flags & RECLS_F_DIRECTORY_PARTS)) ? count_dir_parts(dir0, end) : 0;
    const size_t    cbPath      =   align_up_size(sizeof(char_type) * (1 + cchRootDir + cchFileName));
#if defined(RECLS_PLATFORM_IS_UNIX)
    const size_t    cbAlt       =   0;  // UNIX doesn't have alt paths
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */
    const size_t    cb          =   offsetof(struct recls_fileinfo_t, data)
                                +   cDirParts * sizeof(recls_strptrs_t)
                                +   cbPath
                                +   cbAlt;

    struct recls_fileinfo_t *info   =   const_cast<struct recls_fileinfo_t*>(Entry_Allocate(cb));

    if(NULL != info)
    {
        char_type               *fullPath   =   (char_type*)&info->data[cDirParts * sizeof(recls_strptrs_t)];
#if defined(RECLS_PLATFORM_IS_UNIX)
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */

        // full path
        traits_type::str_copy(&fullPath[0], rootDir);
        traits_type::str_cat(&fullPath[0], entryName);
        info->path.begin                    =   fullPath;
        info->path.end                      =   fullPath + cchRootDir + cchFileName;

        // drive, directory, file (name + ext)
#if defined(RECLS_PLATFORM_IS_UNIX)
        info->directory.begin               =   fullPath;
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */
        info->directory.end                 =   fullPath + cchRootDir;
        info->fileName.begin                =   info->directory.end;
        info->fileName.end                  =   traits_type::str_rchr(info->directory.end, RECLS_LITERAL('.'));
        if(NULL != info->fileName.end)
        {
            info->fileExt.begin             =   info->fileName.end + 1;
            info->fileExt.end               =   info->directory.end +  cchFileName;
        }
        else
        {
            info->fileName.end              =   info->directory.end +  cchFileName;
            info->fileExt.begin             =   info->directory.end +  cchFileName;
            info->fileExt.end               =   info->directory.end +  cchFileName;
        }

        // determine the directory parts
        char_type const         *p          =   info->directory.begin;
        char_type const         *l          =   info->directory.end;
        struct recls_strptrs_t  *begin      =   (struct recls_strptrs_t*)&info->data[0];

        info->directoryParts.begin          =   begin;
        info->directoryParts.end            =   begin + cDirParts;

        if(info->directoryParts.begin != info->directoryParts.end)
        {
            recls_assert((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS);

            begin->begin = p;

            for(; p != l; ++p)
            {
                if(*p == traits_type::path_name_separator())
                {
                    begin->end = p + 1;

                    if(++begin != info->directoryParts.end)
                    {
                        begin->begin = p + 1;
                    }
                }
            }
        }

        // alt name
#if defined(RECLS_PLATFORM_IS_UNIX)
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */

        // attributes
#if defined(RECLS_PLATFORM_IS_UNIX)
        typedef int (*PfnStat)(char const *, struct stat *);

        struct stat st;
#if defined(WIN32) || \
    defined(_WIN32)
        PfnStat     pfn =   stat;
#else /* ? WIN32 */
        PfnStat     pfn =   (RECLS_F_LINKS == (flags & RECLS_F_LINKS)) ? lstat : stat;
#endif /* WIN32 */

        (*pfn)(fullPath, &st);

        info->attributes            =   st.st_mode;
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */

        // time, size
#if defined(RECLS_PLATFORM_IS_UNIX)
        info->lastStatusChangeTime  =   st.st_ctime;
        info->modificationTime      =   st.st_mtime;
        info->lastAccessTime        =   st.st_atime;
        info->size                  =   st.st_size;
#elif defined(RECLS_PLATFORM_IS_WIN32)
#else /* ? platform */
# error Platform not discriminated
#endif /* platform */

        // Checks
        recls_assert(info->path.begin < info->path.end);

        recls_assert(info->directory.begin < info->directory.end);
        recls_assert(info->path.begin <= info->directory.begin);
        recls_assert(info->directory.end <= info->path.end);

        recls_assert(info->fileName.begin <= info->fileName.end);

        recls_assert(info->fileExt.begin <= info->fileExt.end);

        recls_assert(info->fileName.begin < info->fileExt.end);
        recls_assert(info->fileName.end <= info->fileExt.begin);
    }

    return info;
}

#define create_entryinfo_from_psrecord  create_entryinfo_from_psrecord_

/* /////////////////////////////////////////////////////////////////////////////
 * ReclsFileSearchDirectoryNode
 */

ReclsFileSearchDirectoryNode::ReclsFileSearchDirectoryNode(recls_uint32_t flags, ReclsFileSearchDirectoryNode::char_type const *rootDir, ReclsFileSearchDirectoryNode::char_type const *pattern)
    : m_current(NULL)
    , m_dnode(NULL)
    , m_flags(flags)
#ifdef _RECLS_SUPPORTS_MULTIPATTERN
    , m_entries(rootDir, pattern, traits_type::path_separator(), _ssFlags_from_reclsFlags(flags))
#else /* ? _RECLS_SUPPORTS_MULTIPATTERN */
    , m_entries(rootDir, pattern, _ssFlags_from_reclsFlags(flags))
#endif /* _RECLS_SUPPORTS_MULTIPATTERN */
    , m_entriesBegin(m_entries.begin())
#ifdef _MSC_VER // For testing
    , m_directories(rootDir, Recls_GetWildcardsAll(), directory_sequence_type::directories)
#else /* ? _MSC_VER */
    , m_directories(rootDir, traits_type::pattern_all(), directory_sequence_type::directories)
#endif /* _MSC_VER */
#if !defined(RECLS_COMPILER_IS_BORLAND)
    , m_directoriesBegin((flags & RECLS_F_RECURSIVE) ? m_directories.begin() : m_directories.end())
#else
    , m_directoriesBegin(_select_iter((flags & RECLS_F_RECURSIVE), m_directories.begin(), m_directories.end()))
#endif /* !RECLS_COMPILER_IS_BORLAND */
{
    function_scope_trace("ReclsFileSearchDirectoryNode::ReclsFileSearchDirectoryNode");

#if defined(RECLS_COMPILER_IS_BORLAND)
//    m_directoriesBegin = ((flags & RECLS_F_RECURSIVE) ? m_directories.begin() : m_directories.end());
#endif /* !RECLS_COMPILER_IS_BORLAND */

    if(0 == traits_type::str_compare(s_localDir, pattern))
    {
        // This special case handles "." as a search pattern.

        recls_char_t            *file;
        file_path_buffer_type   temp;

        traits_type::str_copy(&temp[0], rootDir);
        traits_type::remove_dir_end(&temp[0]);
        traits_type::get_full_path_name(c_str_ptr(temp), m_rootDir.size(), &m_rootDir[0], &file);
        m_pattern = file;
        *file = '\0';
    }
    else
    {
        traits_type::str_copy(&m_rootDir[0], rootDir);
        traits_type::ensure_dir_end(&m_rootDir[0]);
        m_pattern = pattern;
    }

    recls_assert(stlsoft_raw_offsetof(ReclsFileSearchDirectoryNode, m_entries) < stlsoft_raw_offsetof(ReclsFileSearchDirectoryNode, m_entriesBegin));
    recls_assert(stlsoft_raw_offsetof(ReclsFileSearchDirectoryNode, m_directories) < stlsoft_raw_offsetof(ReclsFileSearchDirectoryNode, m_directoriesBegin));
}

/* static */ ReclsFileSearchDirectoryNode *ReclsFileSearchDirectoryNode::FindAndCreate( recls_uint32_t                                  flags
                                                                                    ,   ReclsFileSearchDirectoryNode::char_type const   *rootDir
                                                                                    ,   ReclsFileSearchDirectoryNode::char_type const   *pattern)
{
    ReclsFileSearchDirectoryNode  *node;

    function_scope_trace("ReclsFileSearchDirectoryNode::FindAndCreate");

#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
    try
    {
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */
        node = new ReclsFileSearchDirectoryNode(flags, rootDir, pattern);
#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
    }
    catch(std::bad_alloc &)
    {
        node = NULL;
    }
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */

    if(NULL != node)
    {
        // Ensure that it, or one of its sub-nodes, has matching entries.
        recls_rc_t  rc = node->Initialise();

        if(RECLS_FAILED(rc))
        {
            delete node;

            node = NULL;
        }
    }

    recls_assert(NULL == node || node->is_valid());

    return node;
}

inline /* static */ ReclsFileSearchDirectoryNode *ReclsFileSearchDirectoryNode::FindAndCreate(  recls_uint32_t                                  flags
                                                                                            ,   ReclsFileSearchDirectoryNode::char_type const   *rootDir
                                                                                            ,   ReclsFileSearchDirectoryNode::char_type const   *subDir
                                                                                            ,   ReclsFileSearchDirectoryNode::char_type const   *pattern)
{
    file_path_buffer_type   compositeDir;

    recls_assert(rootDir[traits_type::str_len(rootDir) - 1] == traits_type::path_name_separator());

#if defined(RECLS_PLATFORM_IS_UNIX)
    // Only need subdir, since globbing provides partial path
    traits_type::str_copy(&compositeDir[0], subDir);
#elif defined(RECLS_PLATFORM_IS_WIN32)
    traits_type::str_copy(&compositeDir[0], rootDir);
    traits_type::str_cat(&compositeDir[0], subDir);
#else /* ? RECLS_PLATFORM_IS_??? */
# error Platform not recognised
#endif /* RECLS_PLATFORM_IS_??? */

    return FindAndCreate(flags, c_str_ptr(compositeDir), pattern);
}

ReclsFileSearchDirectoryNode::~ReclsFileSearchDirectoryNode()
{
    function_scope_trace("ReclsFileSearchDirectoryNode::~ReclsFileSearchDirectoryNode");

    Entry_Release(m_current);

    delete m_dnode;
}

recls_rc_t ReclsFileSearchDirectoryNode::Initialise()
{
    function_scope_trace("ReclsFileSearchDirectoryNode::Initialise");

    recls_rc_t  rc;

    recls_assert(NULL == m_current);
    recls_assert(NULL == m_dnode);

    if(m_entriesBegin != m_entries.end())
    {
        // (i) Try getting a file first,
#if defined(RECLS_PLATFORM_IS_UNIX)
        // The way glob_sequence works
        m_current = create_entryinfo_from_psrecord(c_str_ptr(m_rootDir), m_flags, *m_entriesBegin);
#elif defined(RECLS_PLATFORM_IS_WIN32)
        // The way basic_findfile_sequence<> works
        m_current = create_entryinfo_from_psrecord(c_str_ptr(m_rootDir), m_flags, (*m_entriesBegin).get_find_data());
#else /* ? RECLS_PLATFORM_IS_??? */
# error Platform not recognised
#endif /* RECLS_PLATFORM_IS_??? */

        if(NULL == m_current)
        {
            rc = RECLS_RC_OUT_OF_MEMORY;
        }
        else
        {
            rc = RECLS_RC_OK;
        }
    }
    else
    {
        if(m_directoriesBegin == m_directories.end())
        {
            rc = RECLS_RC_NO_MORE_DATA;
        }
        else
        {
            do
            {
#if defined(RECLS_PLATFORM_IS_UNIX)
                // The way glob_sequence works
                m_dnode = ReclsFileSearchDirectoryNode::FindAndCreate(m_flags, c_str_ptr(m_rootDir), *m_directoriesBegin, c_str_ptr(m_pattern));
#elif defined(RECLS_PLATFORM_IS_WIN32)
                // The way basic_findfile_sequence<> works
                m_dnode = ReclsFileSearchDirectoryNode::FindAndCreate(m_flags, (*m_directoriesBegin).get_path(), c_str_ptr(m_pattern));
#else /* ? RECLS_PLATFORM_IS_??? */
# error Platform not recognised
#endif /* RECLS_PLATFORM_IS_??? */

            } while(NULL == m_dnode && ++m_directoriesBegin != m_directories.end());

            rc = (NULL == m_dnode) ? RECLS_RC_NO_MORE_DATA : RECLS_RC_OK;
        }
    }

    if(RECLS_SUCCEEDED(rc))
    {
        recls_assert(is_valid());
    }

    return rc;
}

recls_bool_t ReclsFileSearchDirectoryNode::is_valid() const
{
    function_scope_trace("ReclsFileSearchDirectoryNode::is_valid");

    recls_rc_t  rc  =   RECLS_RC_OK;

#if !defined(__STLSOFT_COMPILER_IS_MWERKS) && \
    !defined(__STLSOFT_COMPILER_IS_WATCOM)
# ifdef STLSOFT_CF_PRAGMA_MESSAGE_SUPPORT
#  pragma message("Flesh these out")
# endif /* STLSOFT_CF_PRAGMA_MESSAGE_SUPPORT */
    if(RECLS_SUCCEEDED(rc))
    {

    }

    if(RECLS_SUCCEEDED(rc))
    {
    }

    if(RECLS_SUCCEEDED(rc))
    {
    }

    if(RECLS_SUCCEEDED(rc))
    {
    }
#endif /* compiler */

    // (i) Either we are enumerating files (m_current != NULL) or directories (m_dnode != NULL), but not both
    recls_assert(NULL == m_current || NULL == m_dnode);
    // (ii) Either we are enumerating files (m_current != NULL) or there are no more files to enumerate
    recls_assert(NULL != m_current || m_entriesBegin == m_entries.end());

    return RECLS_SUCCEEDED(rc);
}

recls_rc_t ReclsFileSearchDirectoryNode::GetNext()
{
    function_scope_trace("ReclsFileSearchDirectoryNode::GetNext");

    recls_assert(is_valid());

    /* Searching operates as follows:
     *
     * 1. Return all the contents of the files
     * 2. Return the contents of the directories.
     *
     * Hence, if m_dnode is non-NULL, we've already searched
     */

    /* States:
     *
     * - Iterating files: m_entriesBegin != m_entries.end(), m_dnode is NULL, m_directoriesBegin != m_directories.end()
     * - Iterating directories: m_directoriesBegin != m_directories.end(), m_dnode is non-NULL, m_current is NULL
     *
     */

    // Invariants

    // (i) Either we are enumerating files (m_current != NULL) or directories (m_dnode != NULL), but not both
    recls_assert(NULL == m_current || NULL == m_dnode);
    // (ii) Either we are enumerating files (m_current != NULL) or there are no more files to enumerate
    recls_assert(NULL != m_current || m_entriesBegin == m_entries.end());

    recls_rc_t  rc = RECLS_RC_NO_MORE_DATA;

    if(NULL != m_current)
    {
        // Currently enumerating through the files

        recls_assert(m_entriesBegin != m_entries.end());
        recls_assert(NULL == m_dnode);

        // Advance, and check for end of sequence
        ++m_entriesBegin;

        Entry_Release(m_current);
        if(m_entriesBegin != m_entries.end())
        {
            // Still enumerating, so just update m_current
#if defined(RECLS_PLATFORM_IS_UNIX)
            m_current = create_entryinfo_from_psrecord(c_str_ptr(m_rootDir), m_flags, *m_entriesBegin);
#elif defined(RECLS_PLATFORM_IS_WIN32)
            m_current = create_entryinfo_from_psrecord(c_str_ptr(m_rootDir), m_flags, (*m_entriesBegin).get_find_data());
#else /* ? RECLS_PLATFORM_IS_??? */
# error Platform not recognised
#endif /* RECLS_PLATFORM_IS_??? */

            rc = RECLS_RC_OK;
        }
        else
        {
            // No more left in the files sequence, so delete m_current
            m_current = NULL;

            rc = RECLS_RC_NO_MORE_DATA;
        }
    }

    if(NULL == m_current)
    {
        // Now we are either enumerating the directories, or we've already done so

        if(NULL != m_dnode)
        {
            // Currently enumerating the directories
            rc = m_dnode->GetNext();

            if(RECLS_RC_NO_MORE_DATA == rc)
            {
                ++m_directoriesBegin;

                delete m_dnode;

                m_dnode = NULL;
            }
        }

        if(m_directoriesBegin == m_directories.end())
        {
            // Enumeration is complete.
            rc = RECLS_RC_NO_MORE_DATA;
        }
        else
        {
            if(NULL == m_dnode)
            {
                do
                {
                    // Creation of the node will cause it to enter the first enumeration
                    // state. However, if there are no matching

                    recls_assert(m_directoriesBegin != m_directories.end());

#if defined(RECLS_PLATFORM_IS_UNIX)
                    // The way glob_sequence works
                    m_dnode = ReclsFileSearchDirectoryNode::FindAndCreate(m_flags, c_str_ptr(m_rootDir), *m_directoriesBegin, c_str_ptr(m_pattern));
#elif defined(RECLS_PLATFORM_IS_WIN32)
                    // The way basic_findfile_sequence<> works
                    m_dnode = ReclsFileSearchDirectoryNode::FindAndCreate(m_flags, (*m_directoriesBegin).get_path(), c_str_ptr(m_pattern));
#else /* ? RECLS_PLATFORM_IS_??? */
# error Platform not recognised
#endif /* RECLS_PLATFORM_IS_??? */

                    if(NULL != m_dnode)
                    {
                        rc = RECLS_RC_OK;
                    }
                    else
                    {
                        ++m_directoriesBegin;
                    }

                } while(NULL == m_dnode && m_directoriesBegin != m_directories.end());
            }
        }
    }

    recls_assert(is_valid());

    return rc;
}

recls_rc_t ReclsFileSearchDirectoryNode::GetDetails(recls_info_t *pinfo)
{
    function_scope_trace("ReclsFileSearchDirectoryNode::GetDetails");

    recls_assert(is_valid());

    recls_rc_t  rc;

    recls_assert(NULL != pinfo);
    recls_assert(NULL == m_current || NULL == m_dnode);

    if(NULL != m_current)
    {
        // Currently searching for files from the current directory

        recls_assert(NULL == m_dnode);

        rc = Entry_Copy(m_current, pinfo);

#if defined(_RECLS_DEBUG)
        {
            file_path_buffer_type   buffer;

            Recls_GetPathProperty(m_current, &buffer[0], buffer.size());

# if defined(RECLS_CHAR_TYPE_IS_CHAR)
            debug_printf("    [%s]\n", static_cast<char const*>(c_str_ptr(buffer)));
# elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
            debug_printf("    [%s]\n", static_cast<char const*>(winstl::w2a(buffer)));
# endif /* RECLS_CHAR_TYPE_IS_???? */
        }
#endif /* _RECLS_DEBUG */
    }
    else if(NULL != m_dnode)
    {
        recls_assert(NULL == m_current);

        // Sub-directory searching is active, so get from there.

        rc = m_dnode->GetDetails(pinfo);
    }
    else
    {
        // Enumeration has completed
        rc = RECLS_RC_NO_MORE_DATA;
    }

    recls_assert(is_valid());

    return rc;
}

recls_rc_t ReclsFileSearchDirectoryNode::GetNextDetails(recls_info_t *pinfo)
{
    function_scope_trace("ReclsFileSearchDirectoryNode::GetNextDetails");

    recls_assert(is_valid());
    recls_assert(NULL != pinfo);

    recls_rc_t  rc  =   GetNext();

    if(RECLS_SUCCEEDED(rc))
    {
        rc = GetDetails(pinfo);
    }

    recls_assert(is_valid());

    return rc;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
