/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsFileSearch_win32.cpp
 *
 * Purpose:     Implementation of the ReclsFileSearch class for Win32.
 *
 * Created:     16th August 2003
 * Updated:     24th March 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#error This file is now obsolete, as of version 1.6.1. Use ReclsFileSearch.cpp

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls.h>
#include <recls_assert.h>
#include "recls_string.h"
#include "recls_util.h"
#include "recls_impl.h"

#ifdef _RECLS_DEBUG
# include <ctype.h>
#endif /* _RECLS_DEBUG */

#include "recls_debug.h"

#include "ReclsSearch.hpp"
#include "ReclsFileSearch.h"
#include "ReclsFileSearchDirectoryNode.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

namespace {

typedef ReclsFileSearchDirectoryNode::traits_type               traits_type;
typedef ReclsFileSearchDirectoryNode::file_path_buffer_type     file_path_buffer_type;

stlsoft_ns_using(c_str_ptr)

} // anonymous namespace

/* /////////////////////////////////////////////////////////////////////////////
 * Utility functions
 */

inline size_t count_dir_parts(recls_char_t const *begin, recls_char_t const *end)
{
    return count_char_instances(begin, end, traits_type::path_name_separator());
}

/* /////////////////////////////////////////////////////////////////////////////
 * ReclsFileSearch
 */

void *ReclsFileSearch::operator new(size_t cb, int cDirParts, size_t cbRootDir)
{
    function_scope_trace("ReclsFileSearch::operator new");

    cbRootDir = align_up_size(cbRootDir);

    recls_assert(cb > stlsoft_raw_offsetof(ReclsFileSearch, data));

    cb = stlsoft_raw_offsetof(ReclsFileSearch, data)
       + (cDirParts) * sizeof(recls_strptrs_t)
       + cbRootDir;

    void    *pv =   malloc(cb);

#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
    if(NULL == pv)
    {
        throw std::bad_alloc();
    }
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */

    return pv;
}

#ifdef RECLS_COMPILER_REQUIRES_MATCHING_PLACEMENT_DELETE
void ReclsFileSearch::operator delete(void *pv, int /* cDirParts */, size_t /* cbRootDir */)
{
    function_scope_trace("ReclsFileSearch::operator delete");

    free(pv);
}
#endif /* RECLS_COMPILER_REQUIRES_MATCHING_PLACEMENT_DELETE */

void ReclsFileSearch::operator delete(void *pv)
{
    function_scope_trace("ReclsFileSearch::operator delete");

    free(pv);
}

/* static */ recls_rc_t ReclsFileSearch::FindAndCreate( ReclsFileSearch::char_type const    *rootDir
                                                    ,   ReclsFileSearch::char_type const    *pattern
                                                    ,   recls_uint32_t                      flags
                                                    ,   ReclsFileSearch                     **ppsi)
{
    function_scope_trace("ReclsFileSearch::FindAndCreate");

//  debug_printf("Searching: %s / %s / 0x%08x\n", rootDir, pattern, flags);

    recls_rc_t              rc;
    file_path_buffer_type   fullPath;
    size_t                  cchFullPath;

    *ppsi = NULL;

    cchFullPath = traits_type::get_full_path_name(rootDir, fullPath.size(), &fullPath[0]);
    if( 0 == cchFullPath ||
        (traits_type::remove_dir_end(&fullPath[0]), !traits_type::file_exists(c_str_ptr(fullPath))))
    {
        rc = RECLS_RC_INVALID_DIRECTORY;
    }
    else
    {
#if defined(RECLS_PLATFORM_IS_UNIX) && \
    defined(WIN32) && \
    defined(EMULATE_UNIX_ON_WIN32)
        recls_char_t    *fullPath_  =   &fullPath[0];
        recls_char_t    *fullPath   =   fullPath_ + ((fullPath_[0] && fullPath_[1] == ':') ? 2 : 0);

        stlsoft_ns_qual_std(replace)(fullPath, fullPath + cchFullPath, '\\', traits_type::path_name_separator());
#endif /* RECLS_PLATFORM_IS_UNIX && WIN32 && EMULATE_UNIX_ON_WIN32 */

        traits_type::ensure_dir_end(&fullPath[0]);

        size_t      lenSearchRoot   =   traits_type::str_len(c_str_ptr(fullPath));

        recls_assert(0 < lenSearchRoot);

        rootDir = c_str_ptr(fullPath);

        // Count the directory parts. This is always done for the ReclsFileSearch class, since it
        // uses them to recurse.
        char_type const         *dir0       =   find_directory_0(rootDir);
        char_type const *const  end         =   rootDir + lenSearchRoot;
        const int               cDirParts   =   count_dir_parts(dir0, end);

        // Create the search handle
        ReclsFileSearch         *si;

#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
        try
        {
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */
            si = new(cDirParts, sizeof(char_type) * (1 + lenSearchRoot)) ReclsFileSearch(cDirParts, rootDir, pattern, flags);
#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
        }
        catch(std::bad_alloc &)
        {
            si = NULL;
        }
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */

        if(NULL == si)
        {
            rc = RECLS_RC_FAIL;
        }
        else
        {
            // This is a nasty hack. It's tantamount to ctor & create function, so
            // should be made more elegant soon.
            if(NULL == si->m_dnode)
            {
                delete si;

                rc = RECLS_RC_NO_MORE_DATA;
            }
            else
            {
                *ppsi = si;

                rc = RECLS_RC_OK;
            }
        }
    }

//  debug_printf("Searched: rc = 0x%08x; *ppsi = 0x%08x\n", rc, *ppsi);

    recls_assert(RECLS_RC_OK == rc || NULL == *ppsi);

    return rc;
}

ReclsFileSearch::char_type const *ReclsFileSearch::calc_rootDir_(int cDirParts, ReclsFileSearch::char_type const *rootDir)
{
    function_scope_trace("ReclsFileSearch::calc_rootDir_");

    // Root dir is located after file parts, and before pattern
    return traits_type::str_copy(sap_cast<char_type*>(&data[cDirParts * sizeof(recls_strptrs_t)]), rootDir);
}

ReclsFileSearch::ReclsFileSearch(   int                                 cDirParts
                                ,   ReclsFileSearch::char_type const    *rootDir
                                ,   ReclsFileSearch::char_type const    *pattern
                                ,   recls_uint32_t                      flags)
    : m_flags(flags)
    , m_lastError(RECLS_RC_OK)
    , m_rootDir(calc_rootDir_(cDirParts, rootDir))
{
    function_scope_trace("ReclsFileSearch::ReclsFileSearch");

    recls_assert(NULL != rootDir);
    recls_assert(NULL != pattern);
    recls_assert(traits_type::str_len(rootDir) < traits_type::path_max());
    recls_assert(traits_type::str_len(pattern) < traits_type::path_max());

#if defined(RECLS_PLATFORM_IS_WIN32)
    recls_message_assert("Root directory has improper format",  (isalpha(rootDir[0]) && rootDir[1] == ':') || (rootDir[0] == '\\' && rootDir[1] == '\\'));
#elif defined(RECLS_PLATFORM_IS_UNIX)
# if defined(WIN32) && \
     defined(EMULATE_UNIX_ON_WIN32)
    recls_message_assert("Root directory has improper format",  rootDir[0] == '/' || (isalpha(rootDir[0]) && rootDir[1] == ':') || (rootDir[0] == '\\' && rootDir[1] == '\\'));
# else /* ? really unix */
    recls_message_assert("Root directory has improper format", rootDir[0] == '/');
    recls_message_assert("UNIX filesystems do not allow drive specifications", rootDir[1] != ':');
# endif /* ? really unix */
#else /* ? platform*/
# error Platform not recognised
#endif /* platform*/

    // Now start the search
    m_dnode = ReclsFileSearchDirectoryNode::FindAndCreate(m_flags, rootDir, pattern);
}

ReclsFileSearch::~ReclsFileSearch()
{
    delete m_dnode;
}

// Operations
recls_rc_t ReclsFileSearch::GetNext()
{
    function_scope_trace("ReclsFileSearch::GetNext");

    recls_assert(NULL != m_dnode);

    m_lastError =   m_dnode->GetNext();

    if(RECLS_RC_NO_MORE_DATA == m_lastError)
    {
        delete m_dnode;

        m_dnode = NULL;
    }

    return m_lastError;
}

recls_rc_t ReclsFileSearch::GetDetails(recls_info_t *pinfo)
{
    function_scope_trace("ReclsFileSearch::GetDetails");

    recls_assert(NULL != m_dnode);

    return (m_lastError = m_dnode->GetDetails(pinfo));
}

recls_rc_t ReclsFileSearch::GetNextDetails(recls_info_t *pinfo)
{
    function_scope_trace("ReclsFileSearch::GetNextDetails");

    recls_assert(NULL != m_dnode);

    m_lastError =   m_dnode->GetNextDetails(pinfo);

    if(RECLS_RC_NO_MORE_DATA == m_lastError)
    {
        delete m_dnode;

        m_dnode = NULL;
    }

    return m_lastError;
}

// Accessors

recls_rc_t ReclsFileSearch::GetLastError() const
{
    function_scope_trace("ReclsFileSearch::GetLastError");

    return m_lastError;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
