/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_ftp_api_win32.cpp
 *
 * Purpose:     This file contains the Win32 versions of the FTP part of the recls API.
 *
 * Created:     15th August 2004
 * Updated:     20th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls.h>
#include <recls_assert.h>
#include <recls_ftp.h>
#include "recls_string.h"
#include "recls_util.h"
#include "recls_impl.h"

#include <stlsoft.h>
#if !defined(_STLSOFT_VER_1_8_3)
# error STLSoft version 1.8.3 or greater required. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

#if !defined(RECLS_COMPILER_IS_VECTORC) && \
    (   !defined(RECLS_COMPILER_IS_MSVC) || \
        _MSC_VER != 1310)
# include <stlsoft_nulldef.h>
#endif /* compiler */

#include <winstl/file_path_buffer.hpp>
#if defined(RECLS_CHAR_TYPE_IS_WCHAR)
# include <winstl/char_conversions.hpp>
#endif /* RECLS_CHAR_TYPE_IS_???? */

#include <ctype.h>

#include <algorithm>

#include "recls_debug.h"

#if defined(RECLS_DELAY_LOAD_WININET)
# ifdef INETSTL_INCL_H_INETSTL
#  error INETSTL_INCL_H_INETSTL
# endif /* INETSTL_INCL_H_INETSTL */
# include "recls_wininet_dl.h"
#endif /* RECLS_DELAY_LOAD_WININET */

// ftp
#include "ReclsSearch.hpp"
#include "ReclsFtpSearch.hpp"

// win32Registry

#include "EntryFunctions.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

stlsoft_ns_using(c_str_ptr)

/* /////////////////////////////////////////////////////////////////////////////
 * Search control
 */

RECLS_FNDECL(recls_rc_t) Recls_SearchFtp(   recls_char_t const  *host
                                    ,       recls_char_t const  *username
                                    ,       recls_char_t const  *password
                                    ,       recls_char_t const  *searchRoot
                                    ,       recls_char_t const  *pattern
                                    ,       recls_uint32_t      flags
                                    ,       hrecls_t            *phSrch)
{
    function_scope_trace("Recls_SearchFtp");

    recls_assert(NULL != phSrch);

    *phSrch = static_cast<hrecls_t>(0);

    recls_rc_t  rc;

    // Default the input parameters

    // Default the search root
    if( NULL == searchRoot ||
        0 == *searchRoot)
    {
        searchRoot = RECLS_LITERAL("/"); // FTP always rooted at ./.
    }

    // Default the pattern
    if(NULL == pattern)
    {
        pattern = RECLS_LITERAL("*"); // FTP always uses '*' as wildcard
    }

    // Default the flags
    if(0 == (flags & RECLS_F_TYPEMASK))
    {
        flags |= RECLS_F_FILES;
    }

    // Validate the pattern
    rc = IsValidPattern(pattern, flags);

    if(RECLS_SUCCEEDED(rc))
    {
        ReclsFtpSearch *si;

        rc = ReclsFtpSearch::FindAndCreate(host, username, password, searchRoot, pattern, flags, &si);

        if(RECLS_SUCCEEDED(rc))
        {
            *phSrch = ReclsSearch::ToHandle(si);

            rc = RECLS_RC_OK;
        }
    }

    return rc;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
