/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_impl.h
 *
 * Purpose:     Implementation header.
 *
 * Created:     7th March 2005
 * Updated:     20th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted in accordance with the license and warranty
 * information described in recls.h (included in this distribution, or available
 * from http://recls.org/)
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLS_IMPL
#define RECLS_INCL_H_RECLS_IMPL

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_IMPL_MAJOR       2
# define RECLS_VER_H_RECLS_IMPL_MINOR       3
# define RECLS_VER_H_RECLS_IMPL_REVISION    3
# define RECLS_VER_H_RECLS_IMPL_EDIT        22
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_impl.h Implementation header.
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls.h>

#ifdef RECLS_PURE_API
# error Cannot define RECLS_PURE_API for the implementation of recls
#endif /* RECLS_PURE_API */

#if defined(RECLS_PLATFORM_IS_UNIX)
# if defined(WIN32) && \
     defined(EMULATE_UNIX_ON_WIN32)
#  include <windows.h>
# endif /* WIN32 && EMULATE_UNIX_ON_WIN32 */
#endif /* RECLS_PLATFORM_IS_UNIX */

/* /////////////////////////////////////////////////////////////////////////////
 * C++ stuff
 */

#ifdef __cplusplus

#include <stlsoft.h>
#if !defined(_STLSOFT_VER_1_8_3)
# error STLSoft version 1.8.3 or greater required. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

# if !defined(_STLSOFT_VER_1_8_3) || \
    _STLSOFT_VER < _STLSOFT_VER_1_8_3
#  error recls cannot be built with a version of STLSoft prior to 1.8.3
# endif /* _STLSOFT_VER */

# include <stlsoft/sap_cast.hpp>

#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Exceptions
 */

#ifdef __cplusplus
# ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
#  define _RECLS_EXCEPTION_SUPPORT
#  ifdef __STLSOFT_CF_THROW_BAD_ALLOC
#   define RECLS_COMPILER_THROWS_ON_NEW_FAIL
#  endif /* __STLSOFT_CF_THROW_BAD_ALLOC */
# endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Debugging?
 */

/** \def _RECLS_DEBUG If defined, it indicates a debug build */

#if !defined(NDEBUG)
# if defined(RECLS_PLATFORM_IS_UNIX)
#  define _RECLS_DEBUG
# elif defined(RECLS_PLATFORM_IS_WIN32) && \
       (   defined(DEBUG) || \
           defined(_DEBUG))
#  define _RECLS_DEBUG
# endif /* platform */
#endif /* debug */

/* /////////////////////////////////////////////////////////////////////////////
 * Contract enforcement
 */

/** \def RECLS_ENFORCING_CONTRACTS If defined, it indicates that contract enforcement checking will happen */

#if defined(_RECLS_DEBUG) || \
    defined(RECLS_ENFORCE_CONTRACTS)
# define RECLS_ENFORCING_CONTRACTS
#endif /* _RECLS_DEBUG || RECLS_ENFORCE_CONTRACTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Multithreading
 */

/** \def _RECLS_MT If defined, it indicates a multithreaded build */

#if defined(RECLS_PLATFORM_IS_UNIX)
# if defined(_REENTRANT)
#  define _RECLS_MT
# elif defined(WIN32) && \
       defined(EMULATE_UNIX_ON_WIN32) && \
       (    defined(_MT) || \
            defined(__MT__))
#  define _RECLS_MT
#  ifndef _REENTRANT
#   define _REENTRANT
#  endif /* !_REENTRANT */
# endif /* threads? */
#elif defined(RECLS_PLATFORM_IS_WIN32)
# if defined(_MT) || \
     defined(__MT__)
#  define _RECLS_MT
# endif /* threads? */
#endif /* platform */

/* /////////////////////////////////////////////////////////////////////////////
 * RECLS_NUM_ELEMENTS
 */

#ifndef RECLS_NUM_ELEMENTS
# if defined(stlsoft_num_elements)
#  define RECLS_NUM_ELEMENTS(x)             stlsoft_num_elements(x)
# else /* ? stlsoft_num_elements */
#  define RECLS_NUM_ELEMENTS(x)             (sizeof(x) / sizeof((x)[0]))
# endif /* stlsoft_num_elements */
#endif /* !RECLS_NUM_ELEMENTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Multi-part patterns
 */

#if (   !defined(RECLS_COMPILER_IS_MSVC) || \
        _MSC_VER >= 1200) && \
    !defined(RECLS_COMPILER_IS_WATCOM)
# define _RECLS_SUPPORTS_MULTIPATTERN
#endif /* compiler */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS_IMPL */

/* ////////////////////////////////////////////////////////////////////////// */
