/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_util_win32.cpp
 *
 * Purpose:     Win32 utility functions for the recls API.
 *
 * Created:     17th August 2003
 * Updated:     19th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls.h>
#include <recls_assert.h>
#include "recls_impl.h"

#include <unixstl/filesystem_traits.hpp>

#ifdef _RECLS_DEBUG
# include <ctype.h>
#endif /* _RECLS_DEBUG */

#include "recls_debug.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

typedef unixstl_ns_qual(filesystem_traits)<recls_char_t> traits_type;

/* ////////////////////////////////////////////////////////////////////////// */

RECLS_LINKAGE_C recls_char_t const *find_directory_0(recls_char_t const *path)
{
#if defined(WIN32) && \
    defined(EMULATE_UNIX_ON_WIN32)
    if(':' == path[1])
    {
        // It's a drive-prefixed absolute path, so ...
# ifdef _RECLS_DEBUG
        if(!isalpha(path[0]))
        {
            debug_printf("find_directory_0() given an invalid path: %s", path);
        }
# endif /* _RECLS_DEBUG */

        // ... we just skip the drive
        return &path[2];
    }
    else if('\\' == path[0] &&
            '\\' == path[1])
    {
        // It's a UNC absolute path, so we have to find the share name (with a '\')
        // and then the next slash or backslash
        recls_char_t const  *share  =   traits_type::str_chr(path + 2, '\\');

        if(NULL == share)
        {
            goto bad_path_given;
        }
        else
        {
            recls_char_t const  *slash      =   traits_type::str_chr(share + 1, '\\');
            recls_char_t const  *slash_a    =   traits_type::str_chr(share + 1, '/');

            if( NULL == slash ||
                (   NULL != slash_a &&
                    slash_a < slash))
            {
                slash = slash_a;
            }

            if(NULL == slash)
            {
                goto bad_path_given;
            }
            else
            {
                return slash;
            }
        }
    }
    else
    {
        recls_assert(2 < traits_type::str_len(path));

        return path;
    }

bad_path_given:

    // Can't really do _anything_ sensible here, so we just return the
    // end of the string.
# ifdef _RECLS_DEBUG
    debug_printf("find_directory_0() given an invalid path: %s", path);
# endif /* _RECLS_DEBUG */

    return path + traits_type::str_len(path);
#else /* ? WIN32 */
# ifdef _RECLS_DEBUG
    if('/' != path[0])
    {
        debug_printf("find_directory_0() given a non-rooted path: %s", path);
    }
# endif /* _RECLS_DEBUG */

    return path;
#endif /* ? && EMULATE_UNIX_ON_WIN32 */
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
